@@ -271,12 +271,6 @@ bool vtkSMParaViewPipelineController::CreateAnimationHelpers(vtkSMProxy* proxy)
   return true;
 }
 
-//----------------------------------------------------------------------------
-void vtkSMParaViewPipelineController::DoMaterialSetup(vtkSMProxy* vtkNotUsed(proxy))
-{
-  // expected to be overridden by rendering capable subclass
-}
-
 //----------------------------------------------------------------------------
 bool vtkSMParaViewPipelineController::InitializeSession(vtkSMSession* session)
 {
@@ -992,7 +986,7 @@ bool vtkSMParaViewPipelineController::RegisterTextureProxy(vtkSMProxy* proxy, co
     return false;
   }
 
-  SM_SCOPED_TRACE(RegisterTextureProxy).arg("proxy", proxy);
+  SM_SCOPED_TRACE(RegisterTextureProxy).arg("proxy", proxy).arg("filename", filename);
 
   // Register proxies created for proxy list domains.
   this->RegisterProxiesForProxyListDomains(proxy);
@@ -1002,6 +996,34 @@ bool vtkSMParaViewPipelineController::RegisterTextureProxy(vtkSMProxy* proxy, co
   if (filename)
   {
     vtkSMPropertyHelper(proxy, "FileName").Set(filename);
+    vtkSMPropertyHelper(proxy, "Mode").Set(0);
+  }
+  return true;
+}
+
+//----------------------------------------------------------------------------
+bool vtkSMParaViewPipelineController::RegisterTextureProxy(
+  vtkSMProxy* proxy, const char* trivialProducerKey, const char* proxyname)
+{
+  if (!proxy)
+  {
+    return false;
+  }
+
+  SM_SCOPED_TRACE(RegisterTextureProxy)
+    .arg("proxy", proxy)
+    .arg("trivial_producer_key", trivialProducerKey)
+    .arg("proxyname", proxyname);
+
+  // Register proxies created for proxy list domains.
+  this->RegisterProxiesForProxyListDomains(proxy);
+
+  // Register the proxy itself.
+  proxy->GetSessionProxyManager()->RegisterProxy("textures", proxyname, proxy);
+  if (trivialProducerKey)
+  {
+    vtkSMPropertyHelper(proxy, "TrivialProducerKey").Set(trivialProducerKey);
+    vtkSMPropertyHelper(proxy, "Mode").Set(1);
   }
   return true;
 }
