@@ -58,7 +58,8 @@ vtkGLSLModLight::LightStatsBasic vtkGLSLModLight::GetBasicLightStats(
 
 //------------------------------------------------------------------------------
 bool vtkGLSLModLight::ReplaceShaderValues(vtkOpenGLRenderer* renderer, std::string& vertexShader,
-  std::string& /*geometryShader*/, std::string& fragmentShader,
+  std::string& vtkNotUsed(tessControlShader), std::string& vtkNotUsed(tessEvalShader),
+  std::string& vtkNotUsed(geometryShader), std::string& fragmentShader,
   vtkAbstractMapper* vtkNotUsed(mapper), vtkActor* actor)
 {
   vtkShaderProgram::Substitute(
@@ -487,22 +488,18 @@ bool vtkGLSLModLight::ReplaceShaderValues(vtkOpenGLRenderer* renderer, std::stri
       {
         for (int i = 0; i < lastLightCount; ++i)
         {
-          oss << "  L = lightPositionVC" << i
-              << " - vertexVC.xyz;\n"
-                 "  distanceVC = length(L);\n"
-                 "  L = normalize(L);\n"
-                 "  H = normalize(V + L);\n"
-                 "  NdL = clamp(dot(N, L), 1e-5, 1.0);\n"
-                 "  NdH = clamp(dot(N, H), 1e-5, 1.0);\n"
-                 "  HdL = clamp(dot(H, L), 1e-5, 1.0);\n"
-                 "  if (lightPositional"
-              << i
-              << " == 0)\n"
-                 "  {\n"
+          oss << "  if (lightPositional" << i
+              << " == 0) {\n"
                  "    attenuation = 1.0;\n"
-                 "  }\n"
-                 "  else\n"
-                 "  {\n"
+                 "    L = -lightDirectionVC"
+              << i
+              << ";\n"
+                 "  } else {\n"
+                 "    L = lightPositionVC"
+              << i
+              << " - vertexVC.xyz;\n"
+                 "    distanceVC = length(L);\n"
+                 "    L = normalize(L);\n"
                  "    attenuation = 1.0 / (lightAttenuation"
               << i
               << ".x\n"
@@ -534,8 +531,14 @@ bool vtkGLSLModLight::ReplaceShaderValues(vtkOpenGLRenderer* renderer, std::stri
                  "      }\n"
                  "    }\n"
                  "  }\n"
+                 "  H = normalize(V + L);\n"
+                 "  NdL = clamp(dot(N, L), 1e-5, 1.0);\n"
+                 "  NdH = clamp(dot(N, H), 1e-5, 1.0);\n"
+                 "  HdL = clamp(dot(H, L), 1e-5, 1.0);\n"
                  "  radiance = lightColor"
-              << i << " * attenuation;\n";
+              << i
+              << ";\n"
+                 "  radiance *= attenuation;\n";
 
           if (this->UseAnisotropy)
           {
