@@ -416,7 +416,7 @@ int vtkTesting::RegressionTestAndCaptureOutput(double thresh, ostream& os)
 //------------------------------------------------------------------------------
 int vtkTesting::RegressionTest(double thresh)
 {
-  int result = this->RegressionTestAndCaptureOutput(thresh, cout);
+  int result = this->RegressionTest(thresh, cout);
   return result;
 }
 //------------------------------------------------------------------------------
@@ -531,13 +531,17 @@ std::array<double, 3> ComputeMinkowski(vtkDoubleArray* array, double (*f)(double
   {
     for (int dim = 0; dim < 3; ++dim)
     {
+      // The range of ssim values is [-1, 1]. By doing 1 - ssim value, we
+      // change the range to [0, 2]
       measure[dim] += f(1.0 - lab[dim]);
     }
   }
 
+  // Normalize the measure
+  const vtkIdType div = array->GetNumberOfTuples();
   for (int dim = 0; dim < 3; ++dim)
   {
-    measure[dim] /= array->GetNumberOfTuples();
+    measure[dim] /= div;
   }
 
   return measure;
@@ -546,14 +550,15 @@ std::array<double, 3> ComputeMinkowski(vtkDoubleArray* array, double (*f)(double
 //------------------------------------------------------------------------------
 std::array<double, 3> ComputeMinkowski1(vtkDoubleArray* array)
 {
-  return ComputeMinkowski(array, &std::abs);
+  auto same = [](double v) -> double { return v; };
+  return ComputeMinkowski(array, same);
 }
 
 //------------------------------------------------------------------------------
 std::array<double, 3> ComputeMinkowski2(vtkDoubleArray* array)
 {
-  auto f = [](double v) { return v * v; };
-  auto measure = ComputeMinkowski(array, f);
+  auto power2 = [](double v) { return v * v; };
+  auto measure = ComputeMinkowski(array, power2);
   for (int dim = 0; dim < 3; ++dim)
   {
     measure[dim] = std::sqrt(measure[dim]);
@@ -562,34 +567,47 @@ std::array<double, 3> ComputeMinkowski2(vtkDoubleArray* array)
 }
 
 //------------------------------------------------------------------------------
-std::array<double, 3> ComputeWasserstein(vtkDoubleArray* array, double (*f)(double))
+std::array<double, 3> ComputeWasserstein(vtkDoubleArray* array, std::uint64_t (*f)(std::uint64_t))
 {
   std::array<double, 3> measure = {};
 
   auto data = vtk::DataArrayTupleRange<3>(array);
 
-  constexpr int N = 100;
-  std::array<double, N> hist[3] = {};
+  constexpr std::uint64_t N = 200;
+  std::array<std::uint64_t, N> hist[3] = {};
 
   for (auto lab : data)
   {
     for (int dim = 0; dim < 3; ++dim)
     {
-      ++hist[dim][std::round(lab[dim] * (N - 1))];
+      // The range of ssim values is [-1, 1], so we rescale it to [0, 1]
+      double value = (lab[dim] + 1.0) / 2.0;
+      // Find the bucket to place the value in, by rescaling it to [0, N - 1]
+      // [0, (N - 1) / 2] is for negative ssim values,
+      // N / 2 is for ssim = 0,
+      // ((N + 1) / 2, N - 1] is for positive ssim values
+      auto bucket = static_cast<std::uint64_t>(std::round(value * (N - 1)));
+      ++hist[dim][bucket];
     }
   }
 
   for (int dim = 0; dim < 3; ++dim)
   {
-    std::array<double, N> cfd;
+    // Compute the cumulative frequency distribution
+    std::array<std::uint64_t, N> cfd;
     std::partial_sum(hist[dim].begin(), hist[dim].end(), cfd.begin());
 
     for (std::size_t i = 0; i < N - 1; ++i)
     {
       measure[dim] += f(cfd[i]);
     }
+  }
 
-    measure[dim] += f(cfd.back() - array->GetNumberOfTuples());
+  // Normalize the measure
+  const vtkIdType div = f(static_cast<std::uint64_t>(array->GetNumberOfTuples())) * (N - 1);
+  for (int dim = 0; dim < 3; ++dim)
+  {
+    measure[dim] /= div;
   }
 
   return measure;
@@ -598,29 +616,22 @@ std::array<double, 3> ComputeWasserstein(vtkDoubleArray* array, double (*f)(doub
 //------------------------------------------------------------------------------
 std::array<double, 3> ComputeWasserstein1(vtkDoubleArray* array)
 {
-  auto measure = ComputeWasserstein(array, &std::abs);
-  vtkIdType div = array->GetNumberOfTuples() * (100 - 1);
-  for (int dim = 0; dim < 3; ++dim)
-  {
-    measure[dim] /= div;
-  }
-  return measure;
+  auto same = [](std::uint64_t v) -> std::uint64_t { return v; };
+  return ComputeWasserstein(array, same);
 }
 
 //------------------------------------------------------------------------------
 std::array<double, 3> ComputeWasserstein2(vtkDoubleArray* array)
 {
-  auto f = [](double v) { return v * v; };
-  auto measure = ComputeWasserstein(array, f);
-  vtkIdType div = array->GetNumberOfTuples() * array->GetNumberOfTuples() * (100 - 1);
+  auto power2 = [](std::uint64_t v) { return v * v; };
+  auto measure = ComputeWasserstein(array, power2);
   for (int dim = 0; dim < 3; ++dim)
   {
-    measure[dim] /= div;
     measure[dim] = std::sqrt(measure[dim]);
   }
   return measure;
 }
-} // anonymoun namespace
+} // anonymous namespace
 
 //------------------------------------------------------------------------------
 int vtkTesting::RegressionTest(vtkAlgorithm* imageSource, double thresh, ostream& os)
@@ -735,6 +746,7 @@ int vtkTesting::RegressionTest(vtkAlgorithm* imageSource, double thresh, ostream
 
     auto ssim = vtkImageSSIM::New();
     ssim->SetInputToLab();
+    ssim->ClampNegativeValuesOn();
     auto alg = vtkSmartPointer<vtkAlgorithm>::Take(ssim);
     alg->SetInputConnection(pipeline1->GetOutputPort());
     alg->SetInputConnection(1, pipeline2->GetOutputPort());
@@ -908,7 +920,8 @@ int vtkTesting::RegressionTest(vtkAlgorithm* imageSource, double thresh, ostream
     vtkEmscriptenTestUtilities::PreloadDataFile(hostFileName.c_str(), sandboxedFileName);
     // so that subsequent code uses the sandboxed file name instead of host file name.
     delete[] newFileName;
-    newFileName = strdup(sandboxedFileName.c_str());
+    newFileName = new char[sandboxedFileName.size() + 1];
+    strcpy(newFileName, sandboxedFileName.c_str());
 #endif
     if (!LookForFile(newFileName))
     {
@@ -929,7 +942,7 @@ int vtkTesting::RegressionTest(vtkAlgorithm* imageSource, double thresh, ostream
     if ((ext2[1] - ext2[0]) == (ext1[1] - ext1[0]) && (ext2[3] - ext2[2]) == (ext1[3] - ext1[2]) &&
       (ext2[5] - ext2[4]) == (ext1[5] - ext1[4]))
     {
-      vtkLog(INFO, "Trying onother baseline.");
+      vtkLog(INFO, "Trying another baseline.");
       // Cannot compute difference unless image sizes are the same
       executeComparison(error);
     }
@@ -1035,7 +1048,8 @@ int vtkTesting::RegressionTest(vtkAlgorithm* imageSource, double thresh, ostream
 #ifdef __EMSCRIPTEN__
     std::string sandboxedFileName = vtkEmscriptenTestUtilities::PreloadDataFile(newFileName);
     delete[] newFileName;
-    newFileName = strdup(sandboxedFileName.c_str());
+    newFileName = new char[sandboxedFileName.size() + 1];
+    strcpy(newFileName, sandboxedFileName.c_str());
 #endif
     rtPng->SetFileName(newFileName);
     delete[] newFileName;
@@ -1187,7 +1201,7 @@ int vtkTesting::Test(int argc, char* argv[], vtkRenderWindow* rw, double thresh)
   {
     testing->SetRenderWindow(rw);
 
-    return testing->RegressionTestAndCaptureOutput(thresh, cout);
+    return testing->RegressionTest(thresh, cout);
   }
   return NOT_RUN;
 }
